@@ -2,19 +2,19 @@
   File: acl_copy_entry.c
 
   Copyright (C) 1999, 2000
-  Andreas Gruenbacher, <a.gruenbacher@computer.org>
+  Andreas Gruenbacher, <a.gruenbacher@bestbits.at>
 
   This program is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Library General Public
+  modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Library General Public License for more details.
+  Lesser General Public License for more details.
 
-  You should have received a copy of the GNU Library General Public
+  You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
@@ -34,7 +34,7 @@ acl_copy_entry(acl_entry_t dest_d, acl_entry_t src_d)
 	dest_p->etag  = src_p->etag;
 	dest_p->eid   = src_p->eid;
 	dest_p->eperm = src_p->eperm;
-	__acl_reorder_obj_p(dest_p);
+	__acl_reorder_entry_obj_p(dest_p);
 	return 0;
 }
 
