@@ -2,19 +2,19 @@
   File: acl_extended_file.c
 
   Copyright (C) 2000
-  Andreas Gruenbacher, <a.gruenbacher@computer.org>
+  Andreas Gruenbacher, <a.gruenbacher@bestbits.at>
 
   This program is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Library General Public
+  modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Library General Public License for more details.
+  Lesser General Public License for more details.
 
-  You should have received a copy of the GNU Library General Public
+  You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
@@ -34,12 +34,12 @@ acl_extended_file(const char *path_p)
 	int retval;
 
 	retval = getxattr(path_p, ACL_EA_ACCESS, NULL, 0);
-	if (retval < 0 && errno != ENOATTR)
+	if (retval < 0 && errno != ENOATTR && errno != ENODATA)
 		return -1;
 	if (retval > base_size)
 		return 1;
 	retval = getxattr(path_p, ACL_EA_DEFAULT, NULL, 0);
-	if (retval < 0 && errno != ENOATTR)
+	if (retval < 0 && errno != ENOATTR && errno != ENODATA)
 		return -1;
 	if (retval >= base_size)
 		return 1;
